//////////////////////////////////////////////////////////////////////
// default fx functions for Gamestudio Materials
// (c) jcl/Conitec 2006  Version 1.0
// use: #include <transform>,<sun>, etc.
//////////////////////////////////////////////////////////////////////
// jcl 9-4-06 <view> added

ifdef NIX; // prevent that this file is #included
//////////////////////////////////////////////////////////////////////
// often used shader variables & functions

//////////////////////////////////////////////////////////////////////
//section: transform - calculate vertex position /////////////////////

float4x4 matWorldViewProj;
float4 DoTransform(float4 Pos)
{
	return mul(Pos,matWorldViewProj);
}

//////////////////////////////////////////////////////////////////////
//section: sun - return the sun light on the surface /////////////////

float4 vecSunDir;
float4 vecSunColor;
float4 DoSunLight(float3 N)
{
	return vecSunColor * dot(N,-vecSunDir); // modulate sunlight by the surface angle
}

//////////////////////////////////////////////////////////////////////
//section: lights - return the dynamic light on the surface //////////

float4 DoPointLight(float3 P, float3 N, float4 vLightPos, float4 vLightColor)
{
	float3 D = (float3)vLightPos-P; // ray pointing from the light to the surface
#define LIGHTMODULATE
#ifdef LIGHTMODULATE
	float NdotL = dot(N,normalize(D));   // angle between surface and light ray
	float4 vColor = vLightColor * NdotL; // modulate the light by the surface angle
#else
	float NdotL = dot(N,D);
	float4 vColor = vLightColor;
#endif
	
// calculate the light attenuation factor, DX uses a really strange formula here
	float fac = 0.f;
	if (NdotL >= 0.f && vLightPos.w > 0.f)
	{
		float LD = length(D)/vLightPos.w; // get the distance factor
#ifdef DXLIGHTING // falloff formula
		if (LD < 1.3f)
			fac = 1.f/(0.f + 0.f*LD + 1.5f*LD*LD);	
#else  // linear lighting
		if (LD < 1.0f)
			fac = 1.f - LD;
#endif
	}
#ifdef LIGHTMODULATE // increase light to make up for the dimming by the angle
	return vColor * fac * 2.f;
#else	
	return vColor * fac;
#endif	
}

float4 vecLightPos[8];	 // light positions (xyz) and ranges (w)
float4 vecLightColor[8]; // light colors
float4 DoLight(float3 P, float3 N, int i)
{
	return DoPointLight(P,N,vecLightPos[i],vecLightColor[i]);
}

//////////////////////////////////////////////////////////////////////
//section: fog ///////////////////////////////////////////////////////

float4 vecFog;
#ifndef MATWORLD
float4x4 matWorld;
#define MATWORLD
#endif

#ifdef DXFOG
float4x4 matWorldView;
float DoFog(float4 Pos)
{
	float3 P = mul(Pos,matWorldView); // convert vector to view space to get it's depth (.z)
   	return saturate((vecFog.y-P.z) * vecFog.z); // apply the linear fog formula
}
#else // distance based fog
float4 vecViewPos;
float DoFog(float4 Pos)
{
	float3 P = mul(Pos,matWorld);
	return 1 - (distance(P,vecViewPos)-vecFog.x) * vecFog.z;
}
#endif

//////////////////////////////////////////////////////////////////////
//section: pos - transform vertex position to world space ////////////

#ifndef MATWORLD
float4x4 matWorld;
#define MATWORLD
#endif
float4 DoPos(float4 Pos)
{
	return (mul(Pos,matWorld));
}
float3 DoPos(float3 Pos)
{
	return (mul(Pos,matWorld));
}

//////////////////////////////////////////////////////////////////////
//section: view - transform vertex position to view space ////////////

float4x4 matWorldView;
float4 DoView(float4 Pos)
{
	return (mul(Pos,matWorldView));
}
float3 DoView(float3 Pos)
{
	return (mul(Pos,matWorldView));
}

//////////////////////////////////////////////////////////////////////
//section: normal - transform vertex normal //////////////////////////

#ifndef MATWORLD
float4x4 matWorld;
#define MATWORLD
#endif
float3 DoNormal(float3 inNormal)
{
	return normalize(mul(inNormal,matWorld));
}

//////////////////////////////////////////////////////////////////////
//section: tangent - create tangent matrix ///////////////////////////

float3x3 matTangent;	// hint for the engine to create tangents in TEXCOORD2

void CreateTangents(float3 inNormal,float3 inTangent)
{
	matTangent[0] = DoPos(inTangent);
	matTangent[1] = DoPos(cross(inTangent,inNormal));	// binormal
	matTangent[2] = DoPos(inNormal);
}

float3 DoTangent(float3 inVector)
{
	return normalize(mul(matTangent,inVector));
}
//////////////////////////////////////////////////////////////////////
//section: end
endif;
